package de.unibielefeld.cebitec.gi.algogr.exl2.misc;
import java.util.Arrays;

import de.unibielefeld.cebitec.gi.algogr.exl2.HMM;

public class HiddenMarkovSampler<State, Alphabet> {

    private HMM<State, Alphabet> hmm;
    private AliasMethod beginDistRanGen;
    private AliasMethod[] transitionDistRandGen;
    private AliasMethod[] emissionDistRandGen;

    public HiddenMarkovSampler(HMM<State, Alphabet> hmm) {
        this.hmm = hmm;
        this.init();
    }

    public Sample<State, Alphabet> sample(int sequenceLength) {

        Alphabet[] sigma = this.hmm.getAlphabet();
        State[] Q = this.hmm.getStates();

        Alphabet[] sequence = Arrays.copyOf(sigma, sequenceLength);
        State[] path = Arrays.copyOf(Q, sequenceLength);

        int curState = this.beginDistRanGen.choose();
        sequence[0] = sigma[this.emissionDistRandGen[curState].choose()];
        path[0] = Q[curState];

        for (int i = 1; i < sequenceLength; i++) {
            curState = this.transitionDistRandGen[curState].choose();
            sequence[i] = sigma[this.emissionDistRandGen[curState].choose()];
            path[i] = Q[curState];
        }
        return new Sample<State, Alphabet>(sequence, path);
    }

    private void init() {
        int noOfStates = this.hmm.getStates().length;
        double[][] Q = this.hmm.getTransitionMatrix();
        double[][] E = this.hmm.getEmissionMatrix();

        this.beginDistRanGen = new AliasMethod(this.hmm.getBeginDistribution());
        this.transitionDistRandGen = new AliasMethod[noOfStates];
        this.emissionDistRandGen = new AliasMethod[noOfStates];

        for (int k = 0; k < noOfStates; k++) {
            this.transitionDistRandGen[k] = new AliasMethod(Q[k]);
            this.emissionDistRandGen[k] = new AliasMethod(E[k]);
        }
    }

    public static class Sample<State, Alphabet> {

        private Alphabet[] sequence;
        private State[] path;

        Sample(Alphabet[] sequence, State[] path) {
            this.sequence = sequence;
            this.path = path;
        }

        public Alphabet[] getEmissionSequence() {
            return this.sequence;
        }

        public State[] getStatePath() {
            return this.path;
        }
    }
}
