package de.unibielefeld.cebitec.gi.algogr.exl2.misc;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.WindowConstants;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;

public class Utils {

    public static void plotPosteriorDistributionForStateAndPath(
        double[][] distribution, int stateIndex, final Character[] path,
        String name) {

        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        for (int i = 0; i < distribution.length; i++) {
            final String label = path[i].toString();
            final Integer val = i;
            dataset.addValue(distribution[i][stateIndex],
                "Posterior Distribution", new Comparable<Integer>() {

                    @Override
                    public int compareTo(Integer o) {
                        return Integer.compare(o, val);
                    }

                    @Override
                    public String toString() {
                        return label;
                    }
                });
        }

        JFrame frame = new JFrame(name);
        Dimension dim = new Dimension(1000, 600);
        frame.setSize(dim);
        frame.setResizable(true);
        JFreeChart chart = ChartFactory.createLineChart(name, "Path",
            "Posterior Distribution", dataset, PlotOrientation.VERTICAL, false,
            true, true);
        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(dim);
        frame.setContentPane(chartPanel);
        frame.setVisible(true);
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    }

    public static void printEstimatePath(Integer[] sequence,
        Character[] truePath, Character[] estimatedPath) {

        String label1 = "Symbol sequence";
        String label2 = "True path";
        String label3 = "Estimated path";

        int labelOffset = Math.max(label1.length(),
            Math.max(label2.length(), label3.length()));

        for (int i = 0; i < (labelOffset - label1.length()); i++) {
            System.out.print(" ");
        }

        System.out.print(label1 + " ");
        for (int i = 0; i < sequence.length; i++) {
            System.out.print(sequence[i] + " ");
        }
        System.out.println();

        for (int i = 0; i < (labelOffset - label2.length()); i++) {
            System.out.print(" ");
        }

        System.out.print(label2 + " ");
        for (int i = 0; i < truePath.length; i++) {
            System.out.print(truePath[i] + " ");
        }

        System.out.println();
        for (int i = 0; i < (labelOffset - label3.length()); i++) {
            System.out.print(" ");
        }

        System.out.print(label3 + " ");
        int start_i = estimatedPath.length - sequence.length;
        if (start_i < 0) {
            throw new RuntimeException(
                "Symbol sequence is longer than the estimated state path!");
        }
        for (int i = start_i; i < estimatedPath.length; i++) {
            System.out.print(estimatedPath[i] + " ");
        }
    }

}
